# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'form.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFormLayout, QGridLayout,
    QHBoxLayout, QLabel, QLineEdit, QMainWindow,
    QMenu, QMenuBar, QPlainTextEdit, QPushButton,
    QScrollArea, QSizePolicy, QSpacerItem, QStatusBar,
    QTabWidget, QToolButton, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(800, 600)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName(u"actionOpen")
        self.actionSave = QAction(MainWindow)
        self.actionSave.setObjectName(u"actionSave")
        self.actionQuit = QAction(MainWindow)
        self.actionQuit.setObjectName(u"actionQuit")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.gridLayout_4 = QGridLayout(self.centralwidget)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.widget = QWidget(self.centralwidget)
        self.widget.setObjectName(u"widget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy1)
        self.formLayout = QFormLayout(self.widget)
        self.formLayout.setObjectName(u"formLayout")
        self.label = QLabel(self.widget)
        self.label.setObjectName(u"label")

        self.formLayout.setWidget(0, QFormLayout.LabelRole, self.label)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.txtVideo = QLineEdit(self.widget)
        self.txtVideo.setObjectName(u"txtVideo")

        self.horizontalLayout.addWidget(self.txtVideo)

        self.cmdOpenVideo = QToolButton(self.widget)
        self.cmdOpenVideo.setObjectName(u"cmdOpenVideo")

        self.horizontalLayout.addWidget(self.cmdOpenVideo)


        self.formLayout.setLayout(0, QFormLayout.FieldRole, self.horizontalLayout)

        self.label_2 = QLabel(self.widget)
        self.label_2.setObjectName(u"label_2")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.label_2)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.txtAudio = QLineEdit(self.widget)
        self.txtAudio.setObjectName(u"txtAudio")

        self.horizontalLayout_2.addWidget(self.txtAudio)

        self.cmdOpenAudio = QToolButton(self.widget)
        self.cmdOpenAudio.setObjectName(u"cmdOpenAudio")

        self.horizontalLayout_2.addWidget(self.cmdOpenAudio)


        self.formLayout.setLayout(1, QFormLayout.FieldRole, self.horizontalLayout_2)

        self.label_OutDir = QLabel(self.widget)
        self.label_OutDir.setObjectName(u"label_OutDir")

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.label_OutDir)

        self.horizontalLayout_OutDir = QHBoxLayout()
        self.horizontalLayout_OutDir.setObjectName(u"horizontalLayout_OutDir")
        self.txtOutputDir = QLineEdit(self.widget)
        self.txtOutputDir.setObjectName(u"txtOutputDir")

        self.horizontalLayout_OutDir.addWidget(self.txtOutputDir)

        self.cmdOpenOutputDir = QToolButton(self.widget)
        self.cmdOpenOutputDir.setObjectName(u"cmdOpenOutputDir")

        self.horizontalLayout_OutDir.addWidget(self.cmdOpenOutputDir)


        self.formLayout.setLayout(2, QFormLayout.FieldRole, self.horizontalLayout_OutDir)

        self.label_Model = QLabel(self.widget)
        self.label_Model.setObjectName(u"label_Model")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.label_Model)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.cmbModel = QComboBox(self.widget)
        self.cmbModel.addItem("")
        self.cmbModel.addItem("")
        self.cmbModel.addItem("")
        self.cmbModel.addItem("")
        self.cmbModel.setObjectName(u"cmbModel")

        self.horizontalLayout_3.addWidget(self.cmbModel)


        self.formLayout.setLayout(3, QFormLayout.FieldRole, self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.cmbLanguage = QComboBox(self.widget)
        self.cmbLanguage.addItem("")
        self.cmbLanguage.addItem("")
        self.cmbLanguage.setObjectName(u"cmbLanguage")

        self.horizontalLayout_4.addWidget(self.cmbLanguage)


        self.formLayout.setLayout(4, QFormLayout.FieldRole, self.horizontalLayout_4)

        self.label_3 = QLabel(self.widget)
        self.label_3.setObjectName(u"label_3")

        self.formLayout.setWidget(4, QFormLayout.LabelRole, self.label_3)

        self.cmdConvertAudio = QPushButton(self.widget)
        self.cmdConvertAudio.setObjectName(u"cmdConvertAudio")

        self.formLayout.setWidget(6, QFormLayout.FieldRole, self.cmdConvertAudio)

        self.cmdTranscribe = QPushButton(self.widget)
        self.cmdTranscribe.setObjectName(u"cmdTranscribe")

        self.formLayout.setWidget(7, QFormLayout.FieldRole, self.cmdTranscribe)

        self.cmdTerminate = QPushButton(self.widget)
        self.cmdTerminate.setObjectName(u"cmdTerminate")
        self.cmdTerminate.setEnabled(False)

        self.formLayout.setWidget(8, QFormLayout.FieldRole, self.cmdTerminate)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.formLayout.setItem(9, QFormLayout.FieldRole, self.verticalSpacer)


        self.horizontalLayout_5.addWidget(self.widget)

        self.widget_2 = QWidget(self.centralwidget)
        self.widget_2.setObjectName(u"widget_2")
        self.gridLayout_3 = QGridLayout(self.widget_2)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.tabWidget = QTabWidget(self.widget_2)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tabOutput = QWidget()
        self.tabOutput.setObjectName(u"tabOutput")
        self.gridLayout_2 = QGridLayout(self.tabOutput)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(self.tabOutput)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 502, 480))
        self.gridLayout = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout.setObjectName(u"gridLayout")
        self.txtOutput = QPlainTextEdit(self.scrollAreaWidgetContents)
        self.txtOutput.setObjectName(u"txtOutput")

        self.gridLayout.addWidget(self.txtOutput, 0, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout_2.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.tabWidget.addTab(self.tabOutput, "")
        self.tabTxt = QWidget()
        self.tabTxt.setObjectName(u"tabTxt")
        self.gridLayout_2txt = QGridLayout(self.tabTxt)
        self.gridLayout_2txt.setObjectName(u"gridLayout_2txt")
        self.gridLayout_2txt.setContentsMargins(0, 0, 0, 0)
        self.scrollAreatxt = QScrollArea(self.tabTxt)
        self.scrollAreatxt.setObjectName(u"scrollAreatxt")
        self.scrollAreatxt.setWidgetResizable(True)
        self.scrollAreaWidgetContentstxt = QWidget()
        self.scrollAreaWidgetContentstxt.setObjectName(u"scrollAreaWidgetContentstxt")
        self.scrollAreaWidgetContentstxt.setGeometry(QRect(0, 0, 502, 480))
        self.gridLayouttxt = QGridLayout(self.scrollAreaWidgetContentstxt)
        self.gridLayouttxt.setObjectName(u"gridLayouttxt")
        self.txtText = QPlainTextEdit(self.scrollAreaWidgetContentstxt)
        self.txtText.setObjectName(u"txtText")

        self.gridLayouttxt.addWidget(self.txtText, 0, 0, 1, 1)

        self.scrollAreatxt.setWidget(self.scrollAreaWidgetContentstxt)

        self.gridLayout_2txt.addWidget(self.scrollAreatxt, 0, 0, 1, 1)

        self.tabWidget.addTab(self.tabTxt, "")

        self.gridLayout_3.addWidget(self.tabWidget, 0, 0, 1, 1)


        self.horizontalLayout_5.addWidget(self.widget_2)


        self.gridLayout_4.addLayout(self.horizontalLayout_5, 0, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 800, 36))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionQuit)

        self.retranslateUi(MainWindow)
        self.actionOpen.triggered.connect(MainWindow.slotOpen)
        self.cmdOpenAudio.clicked.connect(MainWindow.slotOpenAudio)
        self.cmdOpenVideo.clicked.connect(MainWindow.slotOpenVideo)
        self.cmdConvertAudio.clicked.connect(MainWindow.slotConvertAudio)
        self.cmdTranscribe.clicked.connect(MainWindow.slotTranscribe)
        self.cmdOpenOutputDir.clicked.connect(MainWindow.slotOpenOutputDir)
        self.cmdTerminate.clicked.connect(MainWindow.slotTerminate)
        self.actionQuit.triggered.connect(MainWindow.close)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Whisper GUI", None))
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", u"Open", None))
#if QT_CONFIG(shortcut)
        self.actionOpen.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+O", None))
#endif // QT_CONFIG(shortcut)
        self.actionSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
#if QT_CONFIG(shortcut)
        self.actionSave.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+S", None))
#endif // QT_CONFIG(shortcut)
        self.actionQuit.setText(QCoreApplication.translate("MainWindow", u"Quit", None))
#if QT_CONFIG(shortcut)
        self.actionQuit.setShortcut(QCoreApplication.translate("MainWindow", u"Ctrl+Q", None))
#endif // QT_CONFIG(shortcut)
        self.label.setText(QCoreApplication.translate("MainWindow", u"Video", None))
        self.cmdOpenVideo.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.label_2.setText(QCoreApplication.translate("MainWindow", u"Audio", None))
        self.cmdOpenAudio.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.label_OutDir.setText(QCoreApplication.translate("MainWindow", u"Output Dir", None))
        self.cmdOpenOutputDir.setText(QCoreApplication.translate("MainWindow", u"...", None))
        self.label_Model.setText(QCoreApplication.translate("MainWindow", u"Model", None))
        self.cmbModel.setItemText(0, QCoreApplication.translate("MainWindow", u"tiny", None))
        self.cmbModel.setItemText(1, QCoreApplication.translate("MainWindow", u"medium", None))
        self.cmbModel.setItemText(2, QCoreApplication.translate("MainWindow", u"large", None))
        self.cmbModel.setItemText(3, QCoreApplication.translate("MainWindow", u"small", None))

        self.cmbLanguage.setItemText(0, QCoreApplication.translate("MainWindow", u"German", None))
        self.cmbLanguage.setItemText(1, QCoreApplication.translate("MainWindow", u"English", None))

        self.label_3.setText(QCoreApplication.translate("MainWindow", u"Language", None))
        self.cmdConvertAudio.setText(QCoreApplication.translate("MainWindow", u"Convert Audio", None))
        self.cmdTranscribe.setText(QCoreApplication.translate("MainWindow", u"Transcribe", None))
        self.cmdTerminate.setText(QCoreApplication.translate("MainWindow", u"Terminate", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabOutput), QCoreApplication.translate("MainWindow", u"Output", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabTxt), QCoreApplication.translate("MainWindow", u"Txt", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
    # retranslateUi




import subprocess
from pathlib import Path
from hatchling.builders.hooks.plugin.interface import BuildHookInterface


class UICBuildHook(BuildHookInterface):
    """Build hook to regenerate ui_form.py when form.ui changes."""

    def initialize(self, version, build_data):
        """Initialize the build hook."""
        # Get the source directory
        src_dir = Path("src/whisper_gui")
        ui_file = src_dir / "form.ui"
        py_file = src_dir / "ui_form.py"
        
        # Check if form.ui exists and is newer than ui_form.py
        if ui_file.exists():
            if not py_file.exists() or (
                ui_file.stat().st_mtime > py_file.stat().st_mtime
            ):
                print("Regenerating ui_form.py from form.ui...")
                try:
                    subprocess.run(
                        ["pyside6-uic", str(ui_file), "-o", str(py_file)],
                        check=True,
                        capture_output=True,
                        text=True
                    )
                    print("Successfully regenerated ui_form.py")
                except subprocess.CalledProcessError as e:
                    print(f"Error regenerating ui_form.py: {e.stderr}")
                    raise 


# This Python file uses the following encoding: utf-8
import sys
import os

from PySide6.QtWidgets import QApplication, QMainWindow, QFileDialog, QMessageBox
from PySide6.QtCore import QSettings, QPoint, QSize, QProcess

# Important:
# You need to run the following command to generate the ui_form.py file
#     pyside6-uic form.ui -o ui_form.py, or
#     pyside2-uic form.ui -o ui_form.py
from .ui_form import Ui_MainWindow


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.process = None

        # Enable drag & drop
        self.setAcceptDrops(True)

        self.loadSettings()

        # set self encoding to utf-8 on non-windows and cp1252 on windows
        if sys.platform != "win32":
            self.encoding = "utf-8"
        else:
            self.encoding = "cp1252"

    def closeEvent(self, event):
        self.saveSettings()
        if self.process:
            self.process.terminate()
            self.process.waitForFinished()
        event.accept()

    def loadSettings(self):
        self.settings = QSettings("frank_bergmann", "whisper_gui")
        self.ui.txtAudio.setText(self.settings.value("audio_file", ""))
        self.ui.txtVideo.setText(self.settings.value("video_file", ""))
        self.ui.cmbLanguage.setCurrentText(self.settings.value("language", "German"))
        self.ui.cmbModel.setCurrentText(self.settings.value("model", "tiny"))
        self.ui.txtOutputDir.setText(self.settings.value("output_dir", ""))
        # load position and size
        self.move(self.settings.value("pos", QPoint(200, 200)))
        self.resize(self.settings.value("size", QSize(800, 600)))

    def saveSettings(self):
        self.settings.setValue("audio_file", self.ui.txtAudio.text())
        self.settings.setValue("video_file", self.ui.txtVideo.text())
        self.settings.setValue("language", self.ui.cmbLanguage.currentText())
        self.settings.setValue("model", self.ui.cmbModel.currentText())
        self.settings.setValue("output_dir", self.ui.txtOutputDir.text())
        self.settings.setValue("pos", self.pos())
        self.settings.setValue("size", self.size())
        self.settings.sync()

    def slotTerminate(self):
        print("Terminate")
        self.process.terminate()
        self.process.waitForFinished()
        self.ui.cmdTerminate.setEnabled(False)
        self.ui.cmdTranscribe.setEnabled(True)
        self.ui.cmdConvertAudio.setEnabled(True)

    def slotOpen(self, file_name=None):
        if not file_name:
            file_name, _ = QFileDialog.getOpenFileName(
                self,
                "Open File",
                "",
                "Video Files (*.mp4 *.avi *.mkv *.mov);;Audio Files (*.mp3 *.wav *.m4a *.ogg);;All Files (*.*)"
            )
        if file_name:
            if file_name.endswith(".mp4") or file_name.endswith(".avi") or file_name.endswith(".mkv") or file_name.endswith(".mov"):
                self.ui.txtVideo.setText(file_name)
            elif file_name.endswith(".mp3") or file_name.endswith(".wav") or file_name.endswith(".m4a") or file_name.endswith(".ogg"):
                self.ui.txtAudio.setText(file_name)
        print("Open")

    def slotOpenOutputDir(self):
        print("Open Output Dir")
        output_dir = QFileDialog.getExistingDirectory(
            self,
            "Open Output Dir",
            self.ui.txtOutputDir.text()
        )
        if output_dir:
            self.ui.txtOutputDir.setText(output_dir)

    def slotSave(self):
        print("Save")

    def slotQuit(self):
        print("Quit")

    def slotOpenAudio(self):
        file_name, _ = QFileDialog.getOpenFileName(
            self,
            "Open Audio File",
            self.ui.txtAudio.text(),
            "Audio Files (*.mp3 *.wav *.m4a *.ogg);;All Files (*.*)"
        )
        if file_name:
            self.ui.txtAudio.setText(file_name)
            self.ui.txtVideo.setText("")

    def slotOpenVideo(self):
        file_name, _ = QFileDialog.getOpenFileName(
            self,
            "Open Video File",
            self.ui.txtVideo.text(),
            "Video Files (*.mp4 *.avi *.mkv *.mov);;All Files (*.*)"
        )
        if file_name:
            self.ui.txtVideo.setText(file_name)
            self.ui.txtAudio.setText("")

    def slotConvertAudio(self):
        print("Convert Audio")

        # get the video file
        video_file = self.ui.txtVideo.text()
        if not video_file:
            QMessageBox.warning(self, "Error", "Please select a video file first.")
            return
        
        # create an audio mp3 using ffmpeg
        # target file is original file with different extension
        base_name = video_file[:video_file.rfind(".")] 
        audio_file = base_name + ".mp3"

        # check if audio file already exists
        while os.path.exists(audio_file):
            audio_file = base_name + "_" + str(i) + ".mp3"
            i += 1

        print(f"Converting {video_file} to {audio_file}")

        # run ffmpeg using qprocess
        self.process = QProcess()
        self.process.start("ffmpeg", ["-i", video_file, "-q:a", "0", audio_file])
        self.converted_audio_file = audio_file
        self.process.finished.connect(self.slotConvertAudioFinished)
        self.ui.cmdConvertAudio.setEnabled(False)
        self.ui.cmdTerminate.setEnabled(True)

        self.ui.txtOutput.clear()
        self.ui.txtOutput.appendPlainText("Converting audio...")
        self.ui.txtOutput.appendPlainText("ffmpeg -i " + video_file + " -q:a 0 " + audio_file)

    def slotTranscribe(self):
        print("Transcribe")
        audio_file = self.ui.txtAudio.text()
        if not audio_file:
            QMessageBox.warning(self, "Error", "Please select an audio file first or convert a video file first.")
            return
        
        # run whisper
        self.process = QProcess()

        args = [
            "--language", self.ui.cmbLanguage.currentText(),
            "--model", self.ui.cmbModel.currentText(),
            "--output_format", "txt",
        ]
        if self.ui.txtOutputDir.text():
            args.append("--output_dir")
            args.append(self.ui.txtOutputDir.text())
        args.append(audio_file)

        self.process.start("whisper", args)

        self.ui.txtOutput.clear()
        self.ui.txtOutput.appendPlainText("Transcribing...")
        # add commandline to output
        self.ui.txtOutput.appendPlainText("whisper " + " ".join(args))
        
        self.process.finished.connect(self.slotTranscribeFinished)
        # Connect process signals for progress info
        self.process.readyReadStandardOutput.connect(self.slotTranscribeProgress)
        self.process.readyReadStandardError.connect(self.slotTranscribeError)
        self.ui.cmdTranscribe.setEnabled(False)
        self.ui.cmdTerminate.setEnabled(True)
    
    def slotTranscribeProgress(self):
        print("Transcribe Progress")
        output = self.process.readAllStandardOutput().data().decode(
            encoding=self.encoding,
            errors='replace'
        )
        self.ui.txtOutput.appendPlainText(output)

    def slotTranscribeError(self):
        print("Transcribe Error")
        output = self.process.readAllStandardError().data().decode(
            encoding=self.encoding,
            errors='replace'
        )
        self.ui.txtOutput.appendPlainText(output)

    def slotConvertAudioFinished(self):
        print("Convert Audio Finished")
        self.ui.txtAudio.setText(self.converted_audio_file)
        self.ui.cmdConvertAudio.setEnabled(True)
        self.process = None

    def slotTranscribeFinished(self):
        print("Transcribe Finished")
        output = self.process.readAll().data().decode(
            encoding='utf-8',
            errors='replace'
        )
        self.ui.txtOutput.appendPlainText(output)
        self.ui.cmdTranscribe.setEnabled(True)
        self.process = None

    def dragEnterEvent(self, event):
        """Handle drag enter events."""
        if event.mimeData().hasUrls():
            event.acceptProposedAction()
        else:
            event.ignore()

    def dragMoveEvent(self, event):
        """Handle drag move events."""
        if event.mimeData().hasUrls():
            event.acceptProposedAction()
        else:
            event.ignore()

    def dropEvent(self, event):
        """Handle drop events."""
        files = [url.toLocalFile() for url in event.mimeData().urls()]
        if files:
            # Use the first file dropped
            self.slotOpen(files[0])
            event.acceptProposedAction()
        else:
            event.ignore()


def main():
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    if len(sys.argv) > 1:
        window.slotOpen(sys.argv[1])
    sys.exit(app.exec())


if __name__ == "__main__":
    main()
